use once_cell::sync::Lazy;
use rutie::{class, methods, AnyObject, Float, Hash, Module, NilClass, Object, RString};
use std::{
    cell::RefCell,
    sync::{Arc, Mutex},
};
use sysinfo::{CpuRefreshKind, RefreshKind, System};

// All calls to get system resource information go through a `System` object,
// so keep a single one that is thread-safe
static SYSTEM: Lazy<Arc<Mutex<RefCell<::sysinfo::System>>>> = Lazy::new(|| {
    Arc::new(Mutex::new(RefCell::new(System::new_with_specifics(
        RefreshKind::new().with_cpu(CpuRefreshKind::everything().without_frequency()),
    ))))
});

class!(Sysinfo);
methods!(
    Sysinfo,
    _rtself,
    fn refresh() -> AnyObject {
        let system = SYSTEM.clone();
        let system = system.lock().expect("Couldn't lock"); // TODO
        let mut system = system.borrow_mut();
        system.refresh_cpu();
        NilClass::new().into()
    },
    fn cpu_usage() -> AnyObject {
        let system = SYSTEM.clone();
        let system = system.lock().expect("Couldn't lock"); // TODO
        let mut system = system.borrow_mut();
        let mut data = Hash::new();
        system.refresh_cpu();
        for cpu in system.cpus() {
            let name = cpu.name();
            let name = RString::new_utf8(name);
            let usage = cpu.cpu_usage();
            let usage = Float::new(usage as f64);
            data.store(name, usage);
        }
        data.into()
    }
);

#[no_mangle]
#[allow(non_snake_case)]
pub extern "C" fn Init_sys_info() {
    Module::from_existing("Sys").define(|module| {
        module.get_nested_class("Info").define(|klass| {
            klass.def("cpu_usage", cpu_usage);
            klass.def("refresh", refresh);
        });
    });
}
